#================================================================#
# SKILLS
#================================================================#

import es
import os
from math import sqrt
from etmod_2008.logs.logs import log
from etmod_2008.classes.classes import classes

#___________________________________________CLASSES

# ------------ #
# CLASSES #
# ------------ #
''' Used to represent a skill parameter '''
class SkillParameter:
	def __init__(self, type, price = 0, default = 0, add = 0, max = 0):
		self.Type = type
		self.Price = price
		self.Default = default
		self.Add = add
		self.Max = max
	def CurrentPrice(self, amount, level):
		price = sqrt(self.Price * amount * level)
		if price < self.Price: price = self.Price
		return price
	def SetValue(self, var, val):
		# Set upgrade price
		if var.endswith("price_add"):
			self.Price = int(val)
		# Default value
		elif var.endswith("default"):
			self.Default = int(val)
		# Increment value
		elif var.endswith("add"):
			self.Add = int(val)
		# Max value
		elif var.endswith("max"):
			self.Max = int(val)
	def __str__(self):
		return\
			"Type: " + str(self.Type) + "\n" +\
			"Price: " + str(self.Price) + "\n" +\
			"Default: " + str(self.Default) + "\n" +\
			"Add: " + str(self.Add) + "\n" +\
			"Max: " + str(self.Max) + "\n"
''' Used to represent a skill '''
class Skill:
	def __init__(self, name):
		self.Name = name
		self.Classes = []
		self.Level = 0
		self.CommandString = ""
		self.Price = 0
		self.Parameters = {}
		global skills
		skills[name] = self
	def CurrentPrice(self, amount, level):
		price = sqrt(self.Price * amount * level)
		if price < self.Price: price = self.Price
		return price
	def __str__(self):
		s = "Name: " + self.Name + "\nClasses: ";
		for c in self.Classes:
			s += c.Name + ", "
		s += "\nLevel: " + str(self.Level)
		s += "\nCommand string: " + self.CommandString
		s += "\nPrice: " + str(self.Price)
		s += "\nParameters:\n"
		for p in self.Parameters.values():
			s += str(p)
		s += "\n"
		return s

#___________________________________________DECLARATIONS

# --------------------- #
# DECLARATIONS #
# --------------------- #
# Possible skill parameter types
skillParameterTypes = {"duration" : 0, "intensity" : 1, "health" : 2, "armor" : 3, "speed" : 4, "distance" : 5, "primary" : 6, "secondary" : 7}
skillParameterNames = {0 : "duration", 1 : "intensity", 2 : "health", 3 : "armor", 4 : "speed", 5 : "distance", 6 : "primary", 7 : "secondary"}
# Skills file paths
skillsConfigPath = "cstrike/addons/eventscripts/etmod_2008/skills/configuration"
skillsDataPath = "cstrike/addons/eventscripts/etmod_2008/skills/data"
# Skills
skills = {}

#___________________________________________FUNCTIONS

# ---------------- #
# FUNCTIONS #
# ---------------- #
def loadSkills():
	f = None
	try:
		curPath = os.path.join(os.getcwd(), os.path.join(skillsConfigPath, "skills.cfg"))
		
		f = open(curPath, "r")
		skill = None
		for line in f:
			# Skip commented lines
			if line.startswith("//"):
				continue
				
			# Split based on quotes
			parts = str.split(line, "\"")
			
			# 3 parts = skill name
			if len(parts) == 3:
				log("One skill: " + parts[1])
				skill = Skill(parts[1])
				
			# 5 parts = skill property
			elif len(parts) == 5 and skill is not None:
				var = parts[1]
				val = parts[3].strip("\n")
				val = val.strip("\r")
				log("Skill info: " + var + " - Value: " + val)
				
				# Allowed classes
				if var == "classes":
					cls = val.split(";")
					for c in cls:
						if classes.has_key(c):
							skill.Classes.append(classes[c])
							
				# Required level
				elif var == "level":
					skill.Level = int(val)
					
				# Command string
				elif var == "commandstring":
					skill.CommandString = val
					
				# Price to buy
				elif var == "price_buy":
					skill.Price = int(val)
					
				# Duration parameter
				elif var.startswith("duration"):
					# Add a duration parameter if not yet done
					if not skill.Parameters.has_key("duration"):
						skill.Parameters["duration"] = SkillParameter(skillParameterTypes["duration"])
					# Update param
					skill.Parameters["duration"].SetValue(var, val)
					
				# Intensity parameter
				elif var.startswith("intensity"):
					# Add an intensity parameter if not yet done
					if not skill.Parameters.has_key("intensity"):
						skill.Parameters["intensity"] = SkillParameter(skillParameterTypes["intensity"])
					# Update param
					skill.Parameters["intensity"].SetValue(var, val)
					
				# HP parameter
				elif var.startswith("hp"):
					# Add an intensity parameter if not yet done
					if not skill.Parameters.has_key("health"):
						skill.Parameters["health"] = SkillParameter(skillParameterTypes["health"])
					# Update param
					skill.Parameters["health"].SetValue(var, val)
					
				# AP parameter
				elif var.startswith("ap"):
					# Add an armor parameter if not yet done
					if not skill.Parameters.has_key("armor"):
						skill.Parameters["armor"] = SkillParameter(skillParameterTypes["armor"])
					# Update param
					skill.Parameters["armor"].SetValue(var, val)
				# Speed parameter
				elif var.startswith("speed"):
					# Add a speed parameter if not yet done
					if not skill.Parameters.has_key("speed"):
						skill.Parameters["speed"] = SkillParameter(skillParameterTypes["speed"])
					# Update param
					skill.Parameters["speed"].SetValue(var, val)
				# Distance parameter
				elif var.startswith("distance"):
					# Add a distance parameter if not yet done
					if not skill.Parameters.has_key("distance"):
						skill.Parameters["distance"] = SkillParameter(skillParameterTypes["distance"])
					# Update param
					skill.Parameters["distance"].SetValue(var, val)
				# Primary weapon parameter
				elif var.startswith("primary"):
					# Add a primary parameter if not yet done
					if not skill.Parameters.has_key("primary"):
						skill.Parameters["primary"] = SkillParameter(skillParameterTypes["primary"])
					# Update param
					skill.Parameters["primary"].SetValue(var, val)
				# Secondary weapon parameter
				elif var.startswith("secondary"):
					# Add a secondary parameter if not yet done
					if not skill.Parameters.has_key("secondary"):
						skill.Parameters["secondary"] = SkillParameter(skillParameterTypes["secondary"])
					# Update param
					skill.Parameters["secondary"].SetValue(var, val)
				
		log("Skills config loaded")
	except:
		log("Error while trying to load skills config file")
	finally:
		if f is not None:
			f.close()
			
#___________________________________________SERVER EVENTS

# --------------------- #
# SERVER EVENTS #
# --------------------- #
''' Script loaded '''
def load():
	# Load skills
	loadSkills()
	
	# Log
	log("Skills sub-script loaded")	
''' Script unloaded '''
def unload():
	# Log
	log("Skills sub-script unloaded")
